#!/usr/bin/python

# Copyright (C) 2006, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Based on: http://wiki.laptop.org/go/DCON

import os
import sys
import logging

import dbus
import dbus.glib
import dbus.service
import gobject

HARDWARE_MANAGER_INTERFACE = 'org.laptop.HardwareManager'
HARDWARE_MANAGER_SERVICE = 'org.laptop.HardwareManager'
HARDWARE_MANAGER_OBJECT_PATH = '/org/laptop/HardwareManager'

COLOR_MODE = 0
BLACK_AND_WHITE_MODE = 1
DCON_IS_SOURCE = 0
CPU_IS_SOURCE = 1

_KEYBOARD_BRIGHTNESS_KEY = '/sys/class/leds/olpc:keyboard/brightness'
_DISPLAY_BRIGHTNESS_KEY = '/sys/class/backlight/dcon-bl/brightness'
_DISPLAY_MODE_KEY = '/sys/devices/platform/dcon/output'
_DCON_SOURCE_KEY = '/sys/devices/platform/dcon/source'
_DCON_FREEZE_KEY = '/sys/devices/platform/dcon/freeze'
_POWER_STATE_KEY = '/sys/power/state'

class HardwareManagerService(dbus.service.Object):
    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='i', out_signature='')
    def set_dcon_source(self, source):
        if source != DCON_IS_SOURCE and source != CPU_IS_SOURCE:
            logging.error('Invalid DCON source %d' % source)
            return

        try:
            f = open(_DCON_SOURCE_KEY, 'w')
            f.write(str(source))
            f.close()
        except IOError:
            logging.error('Cannot write DCON source to sysfs.')

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='b', out_signature='')
    def set_dcon_freeze(self, frozen):
        try:
            f = open(_DCON_FREEZE_KEY, 'w')
            if frozen:
                f.write('1')
            else:
                f.write('0')
            f.close()
        except IOError:
            logging.error('Cannot write DCON freeze to sysfs.')

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='i', out_signature='')
    def set_display_mode(self, mode):
        if mode != COLOR_MODE and mode != BLACK_AND_WHITE_MODE:
            logging.error('Invalid mode %d' % mode)
            return

        try:
            f = open(_DISPLAY_MODE_KEY, 'w')
            f.write(str(mode))
            f.close()
        except IOError:
            logging.error('Cannot write DCON mode to sysfs.')

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='i', out_signature='')
    def set_display_brightness(self, level):
        if level < 0 or level > 15:
            return

        try:
            f = open(_DISPLAY_BRIGHTNESS_KEY, 'w')
            f.write(str(level))
            f.close()
        except IOError:
            logging.error('Cannot write DCON backlight level to sysfs.')

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
        in_signature='', out_signature='i')
    def get_display_brightness(self):
        try:
            f = open(_DISPLAY_BRIGHTNESS_KEY, 'r')
            level = int(f.readline())
            f.close()
        except IOError:
            level = -1
            logging.error('Cannot read DCON backlight level from sysfs.')

        return level

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='b', out_signature='')
    def set_keyboard_brightness(self, enabled):
        try:
            f = open(_KEYBOARD_BRIGHTNESS_KEY, 'w')
            if enabled:
                f.write('1')
            else:
                f.write('0')
            f.close()
        except IOError:
            logging.error('Cannot write keyboard brightness to sysfs.')

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
        in_signature='', out_signature='b')
    def get_keyboard_brightness(self):
        try:
            f = open(_KEYBOARD_BRIGHTNESS_KEY, 'r')
            value = f.readline().strip('\n')
            enabled = bool(int(value))
            f.close()
        except IOError:
            enabled = 0
            logging.error('Cannot read keyboard brightness from sysfs.')

        return enabled

    @dbus.service.method(HARDWARE_MANAGER_INTERFACE,
                         in_signature='', out_signature='')
    def set_kernel_suspend(self):
        try:
            f = open(_POWER_STATE_KEY, 'w')
            # 'm' == suspend to memory (RAM)
            f.write('mem')
            f.close()
        except IOError:
            logging.error('Cannot write to kernel power state.')

pid_dir = '/var/run/olpc-hardware-manager/'
if not os.path.isdir(pid_dir):
    try:
        os.mkdir(pid_dir)
    except:
        sys.exit(1)

try:
    pid = os.fork()
    if pid > 0:
        f = open(os.path.join(pid_dir, 'olpc-hardware-manager.pid'),'w')
        f.write('%d' % pid)
        f.close()
        sys.exit(0)
except OSError, e:
    sys.exit(1)

bus = dbus.SystemBus()
bus_name = dbus.service.BusName(HARDWARE_MANAGER_SERVICE, bus=bus)
manager = HardwareManagerService(bus_name, HARDWARE_MANAGER_OBJECT_PATH)

mainloop = gobject.MainLoop()
mainloop.run()
